module Toolchain::Attributes::Helpers
  extend self

  # @param klass [Class]
  #
  # @yield [Symbol] Each defined attribute (key) name.
  #
  def each_key(klass)
    while ![Class, Module, Object, BasicObject, nil].include?(klass)
      klass.keys.each { |key| yield key }
      klass = klass.superclass
    end
  end

  # @param value [Object]
  # @param types [Array<Class>]
  #
  # @return [Boolean] true if the provided value doesn't
  #   match any of the provided type classes.
  #
  def invalid_value?(value, *types)
    value = value.call if value.kind_of?(Proc)

    return false if value.nil?

    types.flatten.each do |type|
      return false if value.kind_of?(type)
    end

    true
  end

  # Converts all keys to Symbol-type, including
  # nested hashes.
  #
  # @param value [Hash]
  # @return [Hash]
  #
  def symbolize_keys(value)
    deep_transform_keys(value) { |key| key.to_sym rescue key }
  end

  # Converts all keys to String-type, including
  # nested hashes.
  #
  # @param value [Hash]
  # @return [Hash]
  #
  def stringify_keys(value)
    deep_transform_keys(value) { |key| key.to_s rescue key }
  end

  private

  # Recursive key transformation method.
  #
  # @param value [Hash]
  # @param block [Proc]
  #
  def deep_transform_keys(value, &block)
    Hash.new.tap do |result|
      value.each do |key, value|
        result[yield(key)] =
          if value.kind_of?(Hash)
            deep_transform_keys(value, &block)
          else
            value
          end
      end
    end
  end
end
