module Toolchain::Validations::Delegator
  module InstanceMethods

    # @param value [Hash]
    #
    attr_writer :errors

    # @return [Hash]
    #
    def errors
      @errors || {}
    end

    # @return [Boolean]
    #
    def valid?
      validator.valid?
    end
  end

  module ClassMethods

    # @param block [Proc]
    #
    # @example
    #   class MyClass
    #     include Toolchain::Validations::Delegator
    #
    #     validate_with do |instance|
    #       if instance.new?
    #         NewResourceValidator
    #       else
    #         ExistingResourceValidator
    #       end
    #     end
    #   end
    #
    def validate_with(&block)
      define_method(:validator) do
        @validator ||= block.call(self).new(self)
      end
    end
  end

  # @param base [Class]
  #
  def self.included(base)
    base.send(:include, InstanceMethods)
    base.extend(ClassMethods)
  end
end
