class Toolchain::Validations::Validators::Base

  # @return [Object] the object that contains the attribute
  #   that'll be validated.
  #
  attr_reader :object

  # @return [Toolchain::Validations::ValidationErrors] the errors object
  #   of the object that's being validated.
  #
  attr_reader :errors

  # @return [Symbol] The key_path of the attribute that's being validated.
  #
  attr_reader :key_path

  # @return [Hash] The value that was passed in to the validator
  #   at definition time. i.e. a Hash containing the regular expression
  #   for Toolchain::Validations::Validators::Formatter along with a message.
  #
  attr_reader :data

  # @return [String] A custom message to override the
  #   default message with in case the attribute is invalid.
  #
  attr_reader :message

  # Instantiates a new Toolchain::Validations::Validators object
  # which'll be used to validate the target object.
  #
  # @param object [Object]
  # @param key_path [Object]
  # @param data [Object]
  #
  def initialize(object, key_path, data)
    @object = object
    @errors = object.errors
    @key_path = key_path
    @data = data
    @message = data[:message] if data.is_a?(Hash)
  end

  # @return [Object] The value that's stored in the attribute of
  #   the object that's being validated.
  #
  def value
    @value ||= (
      keys = key_path.dup
      keys.inject(object.send(keys.shift)) do |memo, key|
        memo[key] if memo.kind_of?(Hash)
      end
    )
  end
end
