module Toolchain::Validations::Validators

  # Validates the format of an email address.
  #
  # @example
  #   class User::Creator
  #     validates :email, email: {
  #       message: "isn't valid"
  #     }
  #   end
  #
  class Email < Base

    PATTERN = /\A[^@\s]+@([^@\s]+\.)+[^@\s]+\z/

    def validate
      errors.add(key_path, message || "is invalid") if no_email?
    end

    private

    def no_email?
      value.nil? || !value.match(PATTERN)
    end
  end
end
