module Toolchain::Validations::Validators

  # Validates the exclusion of values in an attribute.
  # In the example only games without a pegi rating
  # of 12, 16, 18 are valid.
  #
  # @example
  #   class Game::Creator
  #     validates :pegi, exclusion: {
  #       in: [12, 16, 18],
  #       message: "invalid pegi rating"
  #     }
  #   end
  #
  class Exclusion < Base

    def validate
      if included?
        errors.add(key_path, message || "is invalid")
      end
    end

    private

    def included?
      data[:in].include?(value)
    end
  end
end
