module Toolchain::Validations::Validators

  # Validates the format of an attribute using a regular expression.
  #
  # @example
  #   class Company::Creator
  #     validates :credit_card, format: {
  #       with: /^\d+{4}-$\d+{4}-\d+{4}-\d+{4}/,
  #       message: "invalid credit card format"
  #     }
  #   end
  #
  class Format < Base

    def validate
      errors.add(key_path, message || "is invalid") if no_match?
    end

    private

    def no_match?
      value.nil? || !value.match(data[:with])
    end
  end
end
