module Toolchain::Validations::Validators

  # Validates the inclusion of values in an attribute.
  # In the example, only `pending` and `dispatched` are
  # allowed to be set.
  #
  # @example
  #   class Shipment::Creator
  #     validates :status, inclusion: {
  #       in: ["pending", "dispatched"],
  #       message: "available options are: pending, dispatched"
  #     }
  #   end
  #
  class Inclusion < Base

    def validate
      if not_included?
        errors.add(key_path, message || "is invalid")
      end
    end

    private

    def not_included?
      !data[:in].include?(value)
    end
  end
end
