module Toolchain::Validations::Validators

  # Validates the presence of an attribute. If the attribute
  # is either `nil` or `""` (empty String) it is considered invalid.
  #
  # @example
  #   class User::Creator
  #     validates :name, presence: {
  #       message: "is required"
  #     }
  #   end
  #
  class Presence < Base

    def validate
      errors.add(key_path, message || "can't be blank") if blank?
    end

    private

    def blank?
      [nil, ""].include?(value)
    end
  end
end
